/*******************************************************************************
* File Name: SysClkCounter.c  
* Version 2.40
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "SysClkCounter.h"

uint8 SysClkCounter_initVar = 0u;


/*******************************************************************************
* Function Name: SysClkCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void SysClkCounter_Init(void) 
{
        #if (!SysClkCounter_UsingFixedFunction && !SysClkCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!SysClkCounter_UsingFixedFunction && !SysClkCounter_ControlRegRemoved) */
        
        #if(!SysClkCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 SysClkCounter_interruptState;
        #endif /* (!SysClkCounter_UsingFixedFunction) */
        
        #if (SysClkCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            SysClkCounter_CONTROL &= SysClkCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                SysClkCounter_CONTROL2 &= ((uint8)(~SysClkCounter_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                SysClkCounter_CONTROL3 &= ((uint8)(~SysClkCounter_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (SysClkCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                SysClkCounter_CONTROL |= SysClkCounter_ONESHOT;
            #endif /* (SysClkCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            SysClkCounter_CONTROL2 |= SysClkCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            SysClkCounter_RT1 &= ((uint8)(~SysClkCounter_RT1_MASK));
            SysClkCounter_RT1 |= SysClkCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            SysClkCounter_RT1 &= ((uint8)(~SysClkCounter_SYNCDSI_MASK));
            SysClkCounter_RT1 |= SysClkCounter_SYNCDSI_EN;

        #else
            #if(!SysClkCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = SysClkCounter_CONTROL & ((uint8)(~SysClkCounter_CTRL_CMPMODE_MASK));
            SysClkCounter_CONTROL = ctrl | SysClkCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = SysClkCounter_CONTROL & ((uint8)(~SysClkCounter_CTRL_CAPMODE_MASK));
            
            #if( 0 != SysClkCounter_CAPTURE_MODE_CONF)
                SysClkCounter_CONTROL = ctrl | SysClkCounter_DEFAULT_CAPTURE_MODE;
            #else
                SysClkCounter_CONTROL = ctrl;
            #endif /* 0 != SysClkCounter_CAPTURE_MODE */ 
            
            #endif /* (!SysClkCounter_ControlRegRemoved) */
        #endif /* (SysClkCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!SysClkCounter_UsingFixedFunction)
            SysClkCounter_ClearFIFO();
        #endif /* (!SysClkCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        SysClkCounter_WritePeriod(SysClkCounter_INIT_PERIOD_VALUE);
        #if (!(SysClkCounter_UsingFixedFunction && (CY_PSOC5A)))
            SysClkCounter_WriteCounter(SysClkCounter_INIT_COUNTER_VALUE);
        #endif /* (!(SysClkCounter_UsingFixedFunction && (CY_PSOC5A))) */
        SysClkCounter_SetInterruptMode(SysClkCounter_INIT_INTERRUPTS_MASK);
        
        #if (!SysClkCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)SysClkCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            SysClkCounter_WriteCompare(SysClkCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            SysClkCounter_interruptState = CyEnterCriticalSection();
            
            SysClkCounter_STATUS_AUX_CTRL |= SysClkCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(SysClkCounter_interruptState);
            
        #endif /* (!SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void SysClkCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (SysClkCounter_UsingFixedFunction)
        SysClkCounter_GLOBAL_ENABLE |= SysClkCounter_BLOCK_EN_MASK;
        SysClkCounter_GLOBAL_STBY_ENABLE |= SysClkCounter_BLOCK_STBY_EN_MASK;
    #endif /* (SysClkCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!SysClkCounter_ControlRegRemoved || SysClkCounter_UsingFixedFunction)
        SysClkCounter_CONTROL |= SysClkCounter_CTRL_ENABLE;                
    #endif /* (!SysClkCounter_ControlRegRemoved || SysClkCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: SysClkCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  SysClkCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void SysClkCounter_Start(void) 
{
    if(SysClkCounter_initVar == 0u)
    {
        SysClkCounter_Init();
        
        SysClkCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    SysClkCounter_Enable();        
}


/*******************************************************************************
* Function Name: SysClkCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void SysClkCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!SysClkCounter_ControlRegRemoved || SysClkCounter_UsingFixedFunction)
        SysClkCounter_CONTROL &= ((uint8)(~SysClkCounter_CTRL_ENABLE));        
    #endif /* (!SysClkCounter_ControlRegRemoved || SysClkCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (SysClkCounter_UsingFixedFunction)
        SysClkCounter_GLOBAL_ENABLE &= ((uint8)(~SysClkCounter_BLOCK_EN_MASK));
        SysClkCounter_GLOBAL_STBY_ENABLE &= ((uint8)(~SysClkCounter_BLOCK_STBY_EN_MASK));
    #endif /* (SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void SysClkCounter_SetInterruptMode(uint8 interruptsMask) 
{
    SysClkCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: SysClkCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   SysClkCounter_ReadStatusRegister(void) 
{
    return SysClkCounter_STATUS;
}


#if(!SysClkCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: SysClkCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   SysClkCounter_ReadControlRegister(void) 
{
    return SysClkCounter_CONTROL;
}


/*******************************************************************************
* Function Name: SysClkCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    SysClkCounter_WriteControlRegister(uint8 control) 
{
    SysClkCounter_CONTROL = control;
}

#endif  /* (!SysClkCounter_ControlRegRemoved) */


#if (!(SysClkCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: SysClkCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void SysClkCounter_WriteCounter(uint16 counter) \
                                   
{
    #if(SysClkCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (SysClkCounter_GLOBAL_ENABLE & SysClkCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        SysClkCounter_GLOBAL_ENABLE |= SysClkCounter_BLOCK_EN_MASK;
        CY_SET_REG16(SysClkCounter_COUNTER_LSB_PTR, (uint16)counter);
        SysClkCounter_GLOBAL_ENABLE &= ((uint8)(~SysClkCounter_BLOCK_EN_MASK));
    #else
        CY_SET_REG16(SysClkCounter_COUNTER_LSB_PTR, counter);
    #endif /* (SysClkCounter_UsingFixedFunction) */
}
#endif /* (!(SysClkCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: SysClkCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint16) The present value of the counter.
*
*******************************************************************************/
uint16 SysClkCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(SysClkCounter_UsingFixedFunction)
		(void)CY_GET_REG16(SysClkCounter_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(SysClkCounter_COUNTER_LSB_PTR_8BIT);
	#endif/* (SysClkCounter_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(SysClkCounter_UsingFixedFunction)
        return ((uint16)CY_GET_REG16(SysClkCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG16(SysClkCounter_STATICCOUNT_LSB_PTR));
    #endif /* (SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint16) Present Capture value.
*
*******************************************************************************/
uint16 SysClkCounter_ReadCapture(void) 
{
    #if(SysClkCounter_UsingFixedFunction)
        return ((uint16)CY_GET_REG16(SysClkCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG16(SysClkCounter_STATICCOUNT_LSB_PTR));
    #endif /* (SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint16) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void SysClkCounter_WritePeriod(uint16 period) 
{
    #if(SysClkCounter_UsingFixedFunction)
        CY_SET_REG16(SysClkCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG16(SysClkCounter_PERIOD_LSB_PTR, period);
    #endif /* (SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint16) Present period value.
*
*******************************************************************************/
uint16 SysClkCounter_ReadPeriod(void) 
{
    #if(SysClkCounter_UsingFixedFunction)
        return ((uint16)CY_GET_REG16(SysClkCounter_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG16(SysClkCounter_PERIOD_LSB_PTR));
    #endif /* (SysClkCounter_UsingFixedFunction) */
}


#if (!SysClkCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: SysClkCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void SysClkCounter_WriteCompare(uint16 compare) \
                                   
{
    #if(SysClkCounter_UsingFixedFunction)
        CY_SET_REG16(SysClkCounter_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG16(SysClkCounter_COMPARE_LSB_PTR, compare);
    #endif /* (SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint16) Present compare value.
*
*******************************************************************************/
uint16 SysClkCounter_ReadCompare(void) 
{
    return (CY_GET_REG16(SysClkCounter_COMPARE_LSB_PTR));
}


#if (SysClkCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: SysClkCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void SysClkCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    SysClkCounter_CONTROL &= ((uint8)(~SysClkCounter_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    SysClkCounter_CONTROL |= compareMode;
}
#endif  /* (SysClkCounter_COMPARE_MODE_SOFTWARE) */


#if (SysClkCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: SysClkCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void SysClkCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    SysClkCounter_CONTROL &= ((uint8)(~SysClkCounter_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    SysClkCounter_CONTROL |= ((uint8)((uint8)captureMode << SysClkCounter_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (SysClkCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: SysClkCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void SysClkCounter_ClearFIFO(void) 
{

    while(0u != (SysClkCounter_ReadStatusRegister() & SysClkCounter_STATUS_FIFONEMP))
    {
        (void)SysClkCounter_ReadCapture();
    }

}
#endif  /* (!SysClkCounter_UsingFixedFunction) */


/* [] END OF FILE */

