/*******************************************************************************
* File Name: SysClkCounter_PM.c  
* Version 2.40
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "SysClkCounter.h"

static SysClkCounter_backupStruct SysClkCounter_backup;


/*******************************************************************************
* Function Name: SysClkCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysClkCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void SysClkCounter_SaveConfig(void) 
{
    #if (!SysClkCounter_UsingFixedFunction)

        SysClkCounter_backup.CounterUdb = SysClkCounter_ReadCounter();

        #if (CY_UDB_V0)
            SysClkCounter_backup.CounterPeriod = SysClkCounter_ReadPeriod();
            SysClkCounter_backup.CompareValue = SysClkCounter_ReadCompare();
            SysClkCounter_backup.InterruptMaskValue = SysClkCounter_STATUS_MASK;
        #endif /* CY_UDB_V0 */

        #if(!SysClkCounter_ControlRegRemoved)
            SysClkCounter_backup.CounterControlRegister = SysClkCounter_ReadControlRegister();
        #endif /* (!SysClkCounter_ControlRegRemoved) */

    #endif /* (!SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysClkCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void SysClkCounter_RestoreConfig(void) 
{      
    #if (!SysClkCounter_UsingFixedFunction)

        #if (CY_UDB_V0)
            uint8 SysClkCounter_interruptState;
        #endif  /* (CY_UDB_V0) */

       SysClkCounter_WriteCounter(SysClkCounter_backup.CounterUdb);

        #if (CY_UDB_V0)
            SysClkCounter_WritePeriod(SysClkCounter_backup.CounterPeriod);
            SysClkCounter_WriteCompare(SysClkCounter_backup.CompareValue);

            SysClkCounter_interruptState = CyEnterCriticalSection();
            SysClkCounter_STATUS_AUX_CTRL |= SysClkCounter_STATUS_ACTL_INT_EN_MASK;
            CyExitCriticalSection(SysClkCounter_interruptState);

            SysClkCounter_STATUS_MASK = SysClkCounter_backup.InterruptMaskValue;
        #endif  /* (CY_UDB_V0) */

        #if(!SysClkCounter_ControlRegRemoved)
            SysClkCounter_WriteControlRegister(SysClkCounter_backup.CounterControlRegister);
        #endif /* (!SysClkCounter_ControlRegRemoved) */

    #endif /* (!SysClkCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: SysClkCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysClkCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void SysClkCounter_Sleep(void) 
{
    #if(!SysClkCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(SysClkCounter_CTRL_ENABLE == (SysClkCounter_CONTROL & SysClkCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            SysClkCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            SysClkCounter_backup.CounterEnableState = 0u;
        }
    #else
        SysClkCounter_backup.CounterEnableState = 1u;
        if(SysClkCounter_backup.CounterEnableState != 0u)
        {
            SysClkCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!SysClkCounter_ControlRegRemoved) */
    
    SysClkCounter_Stop();
    SysClkCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: SysClkCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  SysClkCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void SysClkCounter_Wakeup(void) 
{
    SysClkCounter_RestoreConfig();
    #if(!SysClkCounter_ControlRegRemoved)
        if(SysClkCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            SysClkCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!SysClkCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
